export declare enum SmoothingAlgorythm {
    Linear = 0,
    BezierPerpendicular = 1,
    CatmullRom = 2,
    BezierWeighted = 3
}
export declare enum ShapeType {
    Circle = 0,
    Line = 1,
    Waveform = 2
}
export declare enum WaveformOrientation {
    Vertical = 0,
    Horizontal = 1
}
export declare enum WaveformShape {
    LineLike = 0,
    Striped = 1
}
export type WaveformOptions = {
    fft_data: Float32Array;
    shape_type: WaveformShape;
    orientation: WaveformOrientation;
};
type Point = {
    x: number;
    y: number;
};
type Shape = {
    shape_type: ShapeType;
    smoothing_algorythm: SmoothingAlgorythm;
    points: Point[];
    waveform_options?: WaveformOptions;
    symmetry?: boolean;
};
export declare class AudioVisual {
    #private;
    constructor(analyzer_node: AnalyserNode, svg_injecting_element: SVGSVGElement, shape: Shape, buffer_length: number, fft_multiplier: number, fft_offset: number, from_fft_range: number, to_fft_range: number, point_count: number);
    on_data(fn: (data: Float32Array) => void): void;
    /**
     * Useful for waveforms or shapes that don't need to redraw every frame
     */
    draw_once(): void;
    draw(): void;
}
export declare class AudioVisualBuilder {
    #private;
    constructor(analyzer_node: AnalyserNode, svg_injecting_element: SVGSVGElement);
    /**
     * The smoothingTimeConstant property of the AnalyserNode interface is a double value representing the averaging constant with the last analysis frame. It's basically an average between the current buffer and the last buffer the AnalyserNode processed, and results in a much smoother set of value changes over time.
     * @param fft_time_smoothing_i A double within the range 0 to 1 (0 meaning no time averaging). The default value is 0.8.
     * @returns this
     */
    set_fft_time_smoothing(fft_time_smoothing_i: number): this;
    /**
     * The fftSize property of the AnalyserNode interface is an unsigned long value and represents the window size in samples that is used when performing a Fast Fourier Transform (FFT) to get frequency domain data.
     *
     *  If the point count set for the analyzer is less than the fft_from > FFT_DATA > fft_to, points might go missing or something will break. In that case increase, otherwise keep to as low as possible due to performance reasons
     * @param fft_size Must be a power of 2 between 2^5 and 2^15, so one of: `32, 64, 128, 256, 512, 1024, 2048, 4096, 8192, 16384, and 32768.` Defaults to `2048`.
     * @returns this
     */
    set_fft_size(fft_size: number): this;
    /**
     * Changes how the FFT data is parsed.
     * Limits the range like:
     *
     * `from_fft_range < FFT_DATA < to_fft_range`,
     *
     * then splits the remaining data evently between point_count.
     * `fft_multiplier` and `fft_offset` define how the data then affects the point mutation. FFT data goes from `-Infinity` to `0` (Noise floor usually around `-160`db from experience) Multiplying the number exaggerates the higher decibel changes, making the result more dynamic, whilst the offset keeps the visualizer from clipping.
     *
     * Reccomended values are around:
     *
     * `shape_type == line`
     *
     *  - multiplier: 3
     *
     *  - offset: -30
     *
     * `shape_type == circle`
     *
     *  - multiplier: 1.5
     *
     *  - offset: 150
     * @param param0
     * @returns
     */
    set_fft_data_tresholds({ from_fft_range_i, to_fft_range_i, point_count_i, fft_multiplier_i, fft_offset_i }: {
        from_fft_range_i?: number | undefined;
        to_fft_range_i?: number | undefined;
        point_count_i?: number | undefined;
        fft_multiplier_i?: number | undefined;
        fft_offset_i?: number | undefined;
    }): this;
    /**
     * Defines what the svg d attribute path command will use.
     *
     * `Linear` - Uses no interpolation between points, jagged but very fast
     *
     * `BezierPerpendicular` - Sets the Cubic Bézier anchors perpendicular to the point. Great for Line shapes with no curves.
     *
     * `CatmullRom` - Uses Centripetal Catmull–Rom spline under the hood, then translates them to Cubic Bézier points. Best quality, worst performance.
     *
     * `BezierWeighted` - DO NOT USE! It's broken at the moment :/. Sets the Cubic Bézier anchors halfway between the next and previous point. Better than Linear on Circular shapes and doesn't have the "invards bulding" side effect of Catmull Rom

     * @param algorythm Linear = 0; BezierPerpendicular = 1; CatmullRom = 2; BezierWeighted = 3;
     * @returns
     */
    set_smoothing_algorythm(algorythm: SmoothingAlgorythm): this;
    /**
     * Creates the Audio Visualizer. To start drawing, use .draw(). To further modify it's looks use CSS, eg:
     * ```css
     * svg {
     *   fill: grey;
     *   stroke: black;
     *   transform: rotate(90deg);
     * }
     * ```
     * @param shape_type Circle = 0; Line = 1;
     * @returns `new AudioVisual`
     */
    build(shape_type: ShapeType, symmetry: boolean, waveform_options?: WaveformOptions): AudioVisual;
}
export {};
